# Python 3.8
"""
Markov Chain

The class MarkovChain allows you to simulate a Markov Chain.
You can use the MCmaker(n) to build a MarkovChain object with random parameters,
for n states.
The example is an instance of MarkovChain.

For using the MarkovChain class, you must pass a row-wise probability matrix "P"
and an initial distribution "d".
"""
import numpy as np

# exporting only: class, instance, builder
__all__ = ["MarkovChain", "example", "MCmaker"]


class MarkovChain:

    def __init__(self, P: np.array, d: np.array):
        """
        This method receives a row-wise Probability Matrix P and an initial
        distribution d and returns an integer generator object,
        which one is a Markov chain state.
        """
        self.__P = np.asarray(P)
        self.__d = np.asarray(d).reshape(1, -1).copy()
        self.__dupdate = self.__d.reshape(1, -1).copy()
        self.__theta = None
        self.__popped = False

    def pop(self):
        """
        It returns an integer regarding to the Markov state at each call.
        """
        if not self.__popped:
            self.__theta = np.where(
                np.random.rand() < np.cumsum(self.__d))[0][0]
            return self.__theta

        self.__theta = np.where(np.random.rand() <
                                np.cumsum(self.__P[self.__theta, :])
                                )[0][0]
        return self.__theta

    def __repr__(self):
        return f"<MarkovChain object, with {self.__d.size} states>"

    def distribution(self, n_steps=1):
        """
        n_steps is the number of steps to forwadly update the distribution. Each call
        of this method makes a new update. For example
            self.distribution()  -> d0
            self.distribution()  -> d1
            self.distribution()  -> d2
            self.distribution(4) -> d6
        """
        if n_steps <= 0:
            return None

        for i in range(n_steps):
            self.__dupdate = self.__dupdate @ self.__P
        self.__dupdate /= np.sum(self.__dupdate)

        return self.__dupdate.copy()

    @property
    def d(self):
        return self.__d.copy()

    @property
    def P(self):
        return self.__P.copy()

    @property
    def state(self):
        if self.__theta is None:
            print("Void! Call .pop() first.")
        return self.__theta

    def reset_distribution(self):
        self.__dupdate = self.__d.copy()

    def reset_state(self):
        self.__popped = False
        self.__theta = None

    def limit_distribution(self, n: int):
        """
        This method resets the distribution to the initial one, and returns
        a sequence of distributions from the initial to the n-th one.
        """
        self.reset_distribution()
        return np.vstack([self.distribution() for i in range(n)])

    def __len__(self):
        return self.d.size

    @property
    def N(self):
        return self.__len__()


def MCmaker(n=3):
    P = np.random.rand(n, n)
    for i in range(n):
        P[i, :] = P[i, :] ** 6
        P[i, :] /= P.sum(axis=1)[i]
    d = np.random.rand(n)
    d /= sum(d)
    return MarkovChain(P, d)


example = MCmaker(np.random.randint(2, 4))




"""
Based on Tkinter, this module offers a class called "AskForColor" that
opens a window for the user to set a color in RGB system.
To get the value in format "#RRGGBB" (used on Tkinter as a pattern),
just use the "get( )" method.
A default color is needed. See the next example:
        from ask_for_color import AskForColor 
        ask = AskForColor("#1a2b3c")
        color = ask.get() # it returns a string color like "#rrggbb"
        numcolor = ask.getnum() # it returns a tuple of int like (12,34,56)

run the command or follow the url
git clone https://github.com/jeykun/AskForColor.git
"""
from setuptools import setup,find_packages
from codecs import open
from os import path
here = path.abspath(path.dirname(__file__))
try:
    with open(path.join(here,'DESCRIPTION.rst'), encoding='utf-8') as f:
        long_description = f.read()
except:
    long_description = ''


#ajuda em https://www.youtube.com/watch?v=Lb0b4ro0ze4&list=PLV7VqBqvsd_0u42lQtg4FU-rWeOSsbzOT&index=5
setup(
    # Ver PEP 426 (name)
    # Iniciar ou terminar com letra ou numero
    name='j5r_mc',
    # Ver PEP 440
    # O formato pode ser assim:
    # 1.2.0.dev1    Development release
    # 1.2.0a1       Alpha Release
    # 1.2.0b1       Beta Release
    # 1.2.0rc1      Release Candidate
    # 1.2.0         Final Release
    # 1.2.0.post1   Post Release
    # 15.10         Date based release
    # 23            Serial release
    version = '0.3',
    description = 'Markov Chain manager.',
    #long_description = 'Using Tkinter module, it allows the user to set any color in RGB system like "#rrggbb".',
    url = '',
    author = 'Junior R. Ribeiro',
    author_email = 'juniorribeiro2013@gmail.com',
    license = 'Free for non-commercial use',
    classifiers = [
        # How mature is this project? Common values are
        #   2   Alpha
        #   4   Beta
        #   5   Production/Stable
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Artistic Software',
        'License :: Free for non-commercial use',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        ],
    keywords = ['Markov','Chain','probability','stochastic'],
    packages = find_packages(),
    install_requires = ['sh>=1.11'],
    data_files = []    
    )
